﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Collections;

namespace SLabelDesign.DesignerControl
{
    public partial class LineControl : UserControl, System.ComponentModel.ICustomTypeDescriptor
    {
        private Color lineColor;
        private int lineWidth;
        private string lineType;
        // 设置属性显示的白名单
        List<string> whiteNameList = new List<string>() { "Size", "Location", "BorderStyle", "LineColor","LineType","LineWidth" }.ConvertAll(d => d.ToLower());

        public LineControl()
        {
            InitializeComponent();
            this.lineColor = this.ForeColor;
            this.lineWidth = 1;
            this.lineType = "水平线";

            AutoDragComponent adcomponent = new AutoDragComponent();
            adcomponent.initProperty(this);

        }

        [Category("其他设置")]
        [Description("直线颜色")]
        [DisplayName("颜色")]
        public Color LineColor                          //进行属性定义，可以保护私有字段
        {
            get
            {
                return this.lineColor;                  //获取属性，返回lineColor
            }
            set
            {
                this.lineColor = value;                 //设置属性，通过value传入
                PaintEventArgs ep = new PaintEventArgs(this.CreateGraphics(), this.ClientRectangle);//用指定的图形和剪辑矩形框来初始化PaintEventArgs对象
                this.LineControl_Paint(this, ep);          //字段修改后，执行Paint方法，这样的话，可以实现对象的重绘
            }
        }

        [Category("其他设置")]
        [Description("直线宽度")]
        [DisplayName("宽度")]
        public int LineWidth                            //同上
        {
            get
            {
                return this.lineWidth;
            }
            set
            {
                this.lineWidth = value;
                PaintEventArgs ep = new PaintEventArgs(this.CreateGraphics(), this.ClientRectangle);
                this.LineControl_Paint(this, ep);
            }
        }

        [Category("其他设置")]
        [Description("直线类型")]
        [DisplayName("直线类型")]
        [TypeConverter(typeof(genderLineTypeItem))]
        public string LineType
        {
            get
            {
                return this.lineType;
            }
            set
            {
                this.lineType = value;
                PaintEventArgs ep = new PaintEventArgs(this.CreateGraphics(), this.ClientRectangle);
                this.LineControl_Paint(this, ep);
            }
        }
        // 生成下拉菜单选项
        public class genderLineTypeItem : StringConverter
        {
            //true enable,false disable
            public override bool GetStandardValuesSupported(ITypeDescriptorContext context)
            {
                return true;
            }
            //true: disable text editting.    false: enable text editting;
            public override bool GetStandardValuesExclusive(ITypeDescriptorContext context)
            {
                return true;
            }
            public override StandardValuesCollection GetStandardValues(ITypeDescriptorContext context)
            {
                return new StandardValuesCollection(new string[] { "水平线", "垂直线" ,"右斜线","左斜线"}); //编辑下拉框中的items
            }

        }

        private void LineControl_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            g.Clear(this.BackColor);
            Pen p = new Pen(this.lineColor);
            p.Width = this.lineWidth * 2;
            //this.Height = this.LineWidth;
            if (LineType.Equals("水平线"))
            {
                //this.Height= this.lineWidth <90? 90 : this.lineWidth;
                g.DrawLine(p, 0, 0, this.Width, 0);
            }
            else if(LineType.Equals("垂直线"))
            {
                //this.Width = this.lineWidth < 90 ? 90 : this.lineWidth;
                g.DrawLine(p, 0, 0, 0, this.Height);
            }
            else if (LineType.Equals("右斜线"))
            {
                //this.Width = this.lineWidth < 90 ? 90 : this.lineWidth;
                g.DrawLine(p, 0, 0, this.Width, this.Height);
            }
            else if (LineType.Equals("左斜线"))
            {
                //this.Width = this.lineWidth < 90 ? 90 : this.lineWidth;
                g.DrawLine(p, this.Width, 0, 0, this.Height);
            }
        }


        private void LineControl_Resize(object sender, EventArgs e)
        {
            PaintEventArgs ep = new PaintEventArgs(this.CreateGraphics(), this.ClientRectangle);
            this.LineControl_Paint(this, ep);
        }

        #region 显示实现接口
        AttributeCollection ICustomTypeDescriptor.GetAttributes()
        {
            return TypeDescriptor.GetAttributes(this, true);
        }

        string ICustomTypeDescriptor.GetClassName()
        {
            return TypeDescriptor.GetClassName(this, true);
        }

        string ICustomTypeDescriptor.GetComponentName()
        {
            return TypeDescriptor.GetComponentName(this, true);
        }

        TypeConverter ICustomTypeDescriptor.GetConverter()
        {
            return TypeDescriptor.GetConverter(this, true);
        }

        EventDescriptor ICustomTypeDescriptor.GetDefaultEvent()
        {
            return TypeDescriptor.GetDefaultEvent(this, true);
        }

        PropertyDescriptor ICustomTypeDescriptor.GetDefaultProperty()
        {
            return TypeDescriptor.GetDefaultProperty(this, true);
        }

        object ICustomTypeDescriptor.GetEditor(Type editorBaseType)
        {
            return TypeDescriptor.GetEditor(this, editorBaseType, true);
        }

        EventDescriptorCollection ICustomTypeDescriptor.GetEvents()
        {
            return TypeDescriptor.GetEvents(this, true);
        }

        EventDescriptorCollection ICustomTypeDescriptor.GetEvents(Attribute[] attributes)
        {
            return TypeDescriptor.GetEvents(this, attributes, true);
        }

        PropertyDescriptorCollection ICustomTypeDescriptor.GetProperties()
        {
            return ((ICustomTypeDescriptor)this).GetProperties(new Attribute[0]);
        }

        PropertyDescriptorCollection ICustomTypeDescriptor.GetProperties(Attribute[] attributes)
        {
            ArrayList props = new ArrayList();
            Type thisType = this.GetType();

            PropertyDescriptorCollection propsDC = TypeDescriptor.GetProperties(thisType);

            foreach (PropertyDescriptor dc in propsDC)
            {
                if (whiteNameList.Contains(dc.Name.ToLower()))
                {
                    props.Add(dc);
                }
            }

            PropertyDescriptor[] propArray =
            (PropertyDescriptor[])props.ToArray(typeof(PropertyDescriptor));
            return new PropertyDescriptorCollection(propArray);
        }

        object ICustomTypeDescriptor.GetPropertyOwner(PropertyDescriptor pd)
        {
            return this;
        }


        #endregion


    }



    [Serializable]
    public class LineControlModel
    {
        public Size CtlSize { get; set; }
        public Point CtlLocation { get; set; }
        public BorderStyle BorderStyle { get; set; }
        public Color LineColor { get; set; }
        public int LineWidth { get; set; }
        public string LineType { get; set; }

    }

}
