﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Collections;

namespace SLabelDesign.DesignerControl
{
    public partial class PictureControl : System.Windows.Forms.PictureBox, System.ComponentModel.ICustomTypeDescriptor
    {

        // 设置属性显示的白名单
        List<string> whiteNameList = new List<string>() { "Size","Location","BorderStyle", "Image", "SizeMode" }.ConvertAll(d => d.ToLower());

        public PictureControl()
        {
            AutoDragComponent adcomponent = new AutoDragComponent();
            adcomponent.initProperty(this);

            this.BorderStyle = BorderStyle.FixedSingle;
            this.Width = 100;
            this.Height = 100;
            this.SizeMode = PictureBoxSizeMode.StretchImage;
            this.Location = new Point(10, 10);
        }

        #region 显示实现接口
        AttributeCollection ICustomTypeDescriptor.GetAttributes()
        {
            return TypeDescriptor.GetAttributes(this, true);
        }

        string ICustomTypeDescriptor.GetClassName()
        {
            return TypeDescriptor.GetClassName(this, true);
        }

        string ICustomTypeDescriptor.GetComponentName()
        {
            return TypeDescriptor.GetComponentName(this, true);
        }

        TypeConverter ICustomTypeDescriptor.GetConverter()
        {
            return TypeDescriptor.GetConverter(this, true);
        }

        EventDescriptor ICustomTypeDescriptor.GetDefaultEvent()
        {
            return TypeDescriptor.GetDefaultEvent(this, true);
        }

        PropertyDescriptor ICustomTypeDescriptor.GetDefaultProperty()
        {
            return TypeDescriptor.GetDefaultProperty(this, true);
        }

        object ICustomTypeDescriptor.GetEditor(Type editorBaseType)
        {
            return TypeDescriptor.GetEditor(this, editorBaseType, true);
        }

        EventDescriptorCollection ICustomTypeDescriptor.GetEvents()
        {
            return TypeDescriptor.GetEvents(this, true);
        }

        EventDescriptorCollection ICustomTypeDescriptor.GetEvents(Attribute[] attributes)
        {
            return TypeDescriptor.GetEvents(this, attributes, true);
        }

        PropertyDescriptorCollection ICustomTypeDescriptor.GetProperties()
        {
            return ((ICustomTypeDescriptor)this).GetProperties(new Attribute[0]);
        }

        PropertyDescriptorCollection ICustomTypeDescriptor.GetProperties(Attribute[] attributes)
        {
            ArrayList props = new ArrayList();
            Type thisType = this.GetType();

            PropertyDescriptorCollection propsDC = TypeDescriptor.GetProperties(thisType);

            foreach (PropertyDescriptor dc in propsDC)
            {
                if (whiteNameList.Contains(dc.Name.ToLower()))
                {
                    props.Add(dc);
                }
            }

            PropertyDescriptor[] propArray =
            (PropertyDescriptor[])props.ToArray(typeof(PropertyDescriptor));
            return new PropertyDescriptorCollection(propArray);
        }

        object ICustomTypeDescriptor.GetPropertyOwner(PropertyDescriptor pd)
        {
            return this;
        }


        #endregion

    }

    [Serializable]
    public class PictureControlModel
    {
        public Size CtlSize { get; set; }
        public Point CtlLocation { get; set; }
        public BorderStyle CtlBorderStyle { get; set; }
        public String CtlImage { get; set; }
        public PictureBoxSizeMode CtlSizeMode { get; set; }

    }

}
