﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ZXing;
using ZXing.QrCode;

namespace SLabelDesign.DesignerControl
{


    public partial class QRCodeControl : PictureBox, System.ComponentModel.ICustomTypeDescriptor
    {
        private string _QRCodeStr="1234567890";
        private string _QRCodeType = "QRCode";
        // 设置属性显示的白名单
        List<string> whiteNameList = new List<string>() { "Size", "Location", "BorderStyle", "QRCodeType", "QRCodeStr" }.ConvertAll(d => d.ToLower());

        public QRCodeControl()
        {
            AutoDragComponent adcomponent = new AutoDragComponent();
            adcomponent.initProperty(this);

            double MonitorDPIX = 1.0f;
            double MonitorDPIY = 1.0f;

            using (Graphics graphics = Graphics.FromHwnd(IntPtr.Zero))
            {
                MonitorDPIX = graphics.DpiX;
                MonitorDPIY = graphics.DpiY;
            }

            this.SizeMode = PictureBoxSizeMode.StretchImage;
            this.Width = 100;
            this.Height = 100;

            this.SizeChanged += (object obj, EventArgs args) =>
            {
                this.Image = GenerateQRCode(this.QRCodeStr, this.Width, this.Height,QRCodeType);
                this.SizeMode = PictureBoxSizeMode.StretchImage;
            };

        }

        #region 属性设置
        [Category("其他设置")]
        [Description("条码类型设置")]
        [DisplayName("条码类型")]
        [TypeConverter(typeof(genderQRCodeStrItem))]
        public string QRCodeType {
            get { return this._QRCodeType; }
            set {
                _QRCodeType = value;
                this.Image = GenerateQRCode(QRCodeStr, this.Width, this.Height, value);
            }
        }


        [Category("其他设置")]
        [Description("条码内容设置")]
        [DisplayName("条码内容")]
        public string QRCodeStr
        {
            get { return _QRCodeStr; }
            set
            {
                this._QRCodeStr = value;
                this.Image = GenerateQRCode(value, this.Width, this.Height,QRCodeType);
                 
            }
        }

        #endregion

        // 生成下拉菜单选项
        public class genderQRCodeStrItem : StringConverter
        {
            //true enable,false disable
            public override bool GetStandardValuesSupported(ITypeDescriptorContext context)
            {
                return true;
            }
            //true: disable text editting.    false: enable text editting;
            public override bool GetStandardValuesExclusive(ITypeDescriptorContext context)
            {
                return true;
            }
            public override StandardValuesCollection GetStandardValues(ITypeDescriptorContext context)
            {
                return new StandardValuesCollection(new string[] { "QRCode", "DATA MATRIX" }); //编辑下拉框中的items
            }

        }

        // 生成二维码
        private Bitmap GenerateQRCode(string text, int width, int height,string type)
        {
            BarcodeWriter writer = new BarcodeWriter();
            switch (type)
            {
                case "QRCode":
                    writer.Format = BarcodeFormat.QR_CODE;
                    break;
                case "DATA MATRIX":
                    writer.Format = BarcodeFormat.DATA_MATRIX;
                    break;
                default:
                    writer.Format = BarcodeFormat.QR_CODE;
                    break;
            }
            
            
            QrCodeEncodingOptions options = new QrCodeEncodingOptions()
            {
                DisableECI = true,
                CharacterSet = "UTF-8",
                Width = width,
                Height = height,
                Margin = 0,
            };
            writer.Options = options;

            Bitmap map = writer.Write(text);
            return map;
        }

        #region 显示实现接口
        AttributeCollection ICustomTypeDescriptor.GetAttributes()
        {
            return TypeDescriptor.GetAttributes(this, true);
        }

        string ICustomTypeDescriptor.GetClassName()
        {
            return TypeDescriptor.GetClassName(this, true);
        }

        string ICustomTypeDescriptor.GetComponentName()
        {
            return TypeDescriptor.GetComponentName(this, true);
        }

        TypeConverter ICustomTypeDescriptor.GetConverter()
        {
            return TypeDescriptor.GetConverter(this, true);
        }

        EventDescriptor ICustomTypeDescriptor.GetDefaultEvent()
        {
            return TypeDescriptor.GetDefaultEvent(this, true);
        }

        PropertyDescriptor ICustomTypeDescriptor.GetDefaultProperty()
        {
            return TypeDescriptor.GetDefaultProperty(this, true);
        }

        object ICustomTypeDescriptor.GetEditor(Type editorBaseType)
        {
            return TypeDescriptor.GetEditor(this, editorBaseType, true);
        }

        EventDescriptorCollection ICustomTypeDescriptor.GetEvents()
        {
            return TypeDescriptor.GetEvents(this, true);
        }

        EventDescriptorCollection ICustomTypeDescriptor.GetEvents(Attribute[] attributes)
        {
            return TypeDescriptor.GetEvents(this, attributes, true);
        }

        PropertyDescriptorCollection ICustomTypeDescriptor.GetProperties()
        {
            return ((ICustomTypeDescriptor)this).GetProperties(new Attribute[0]);
        }

        PropertyDescriptorCollection ICustomTypeDescriptor.GetProperties(Attribute[] attributes)
        {
            ArrayList props = new ArrayList();
            Type thisType = this.GetType();

            PropertyDescriptorCollection propsDC = TypeDescriptor.GetProperties(thisType);

            foreach(PropertyDescriptor dc in propsDC)
            {
                if(whiteNameList.Contains(dc.Name.ToLower()))
                {
                    props.Add(dc);
                }
            }

            PropertyDescriptor[] propArray =
            (PropertyDescriptor[])props.ToArray(typeof(PropertyDescriptor));
            return new PropertyDescriptorCollection(propArray);
        }

        object ICustomTypeDescriptor.GetPropertyOwner(PropertyDescriptor pd)
        {
            return this;
        }


        #endregion

    }

    [Serializable]
    public class QRCodeControlModel
    {
        public Size CtlSize { get; set; }
        public Point CtlLocation { get; set; }
        public string QRCodeStr { get; set; }
        public string QRCodeType { get; set; }
        public BorderStyle BorderStyle { get; set; }
    }

}
