﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Collections;

namespace SLabelDesign.DesignerControl
{
    public partial class RectControl : UserControl, System.ComponentModel.ICustomTypeDescriptor
    {
        private Color lineColor;
        private int lineWidth;


        // 设置属性显示的白名单
        List<string> whiteNameList = new List<string>() { "Size", "Location", "BorderStyle", "LineColor", "LineWidth" }.ConvertAll(d => d.ToLower());

        public RectControl()
        {
            InitializeComponent();
            this.lineColor = this.ForeColor;
            this.lineWidth = 1;

            AutoDragComponent adcomponent = new AutoDragComponent();
            adcomponent.initProperty(this);
        }

        [Category("其他设置")]
        [Description("直线颜色")]
        [DisplayName("颜色")]
        public Color LineColor                          //进行属性定义，可以保护私有字段
        {
            get
            {
                return this.lineColor;                  //获取属性，返回lineColor
            }
            set
            {
                this.lineColor = value;                 //设置属性，通过value传入
                PaintEventArgs ep = new PaintEventArgs(this.CreateGraphics(), this.ClientRectangle);//用指定的图形和剪辑矩形框来初始化PaintEventArgs对象
                this.RectUserControl_Paint(this, ep);          //字段修改后，执行Paint方法，这样的话，可以实现对象的重绘
            }
        }

        [Category("其他设置")]
        [Description("直线宽度")]
        [DisplayName("宽度")]
        public int LineWidth                            //同上
        {
            get
            {
                return this.lineWidth;
            }
            set
            {
                this.lineWidth = value;
                PaintEventArgs ep = new PaintEventArgs(this.CreateGraphics(), this.ClientRectangle);
                this.RectUserControl_Paint(this, ep);
            }
        }
        private void RectUserControl_Resize(object sender, EventArgs e)
        {
            PaintEventArgs ep = new PaintEventArgs(this.CreateGraphics(), this.ClientRectangle);
            this.RectUserControl_Paint(this, ep);
        }
        private void RectUserControl_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            g.Clear(this.BackColor);
            Pen p = new Pen(this.lineColor);
            p.Width = this.lineWidth * 2;

            g.DrawRectangle(p, 0,0,this.Width,this.Height);

        }

        #region 显示实现接口
        AttributeCollection ICustomTypeDescriptor.GetAttributes()
        {
            return TypeDescriptor.GetAttributes(this, true);
        }

        string ICustomTypeDescriptor.GetClassName()
        {
            return TypeDescriptor.GetClassName(this, true);
        }

        string ICustomTypeDescriptor.GetComponentName()
        {
            return TypeDescriptor.GetComponentName(this, true);
        }

        TypeConverter ICustomTypeDescriptor.GetConverter()
        {
            return TypeDescriptor.GetConverter(this, true);
        }

        EventDescriptor ICustomTypeDescriptor.GetDefaultEvent()
        {
            return TypeDescriptor.GetDefaultEvent(this, true);
        }

        PropertyDescriptor ICustomTypeDescriptor.GetDefaultProperty()
        {
            return TypeDescriptor.GetDefaultProperty(this, true);
        }

        object ICustomTypeDescriptor.GetEditor(Type editorBaseType)
        {
            return TypeDescriptor.GetEditor(this, editorBaseType, true);
        }

        EventDescriptorCollection ICustomTypeDescriptor.GetEvents()
        {
            return TypeDescriptor.GetEvents(this, true);
        }

        EventDescriptorCollection ICustomTypeDescriptor.GetEvents(Attribute[] attributes)
        {
            return TypeDescriptor.GetEvents(this, attributes, true);
        }

        PropertyDescriptorCollection ICustomTypeDescriptor.GetProperties()
        {
            return ((ICustomTypeDescriptor)this).GetProperties(new Attribute[0]);
        }

        PropertyDescriptorCollection ICustomTypeDescriptor.GetProperties(Attribute[] attributes)
        {
            ArrayList props = new ArrayList();
            Type thisType = this.GetType();

            PropertyDescriptorCollection propsDC = TypeDescriptor.GetProperties(thisType);

            foreach (PropertyDescriptor dc in propsDC)
            {
                if (whiteNameList.Contains(dc.Name.ToLower()))
                {
                    props.Add(dc);
                }
            }

            PropertyDescriptor[] propArray =
            (PropertyDescriptor[])props.ToArray(typeof(PropertyDescriptor));
            return new PropertyDescriptorCollection(propArray);
        }

        object ICustomTypeDescriptor.GetPropertyOwner(PropertyDescriptor pd)
        {
            return this;
        }


        #endregion




    }


    [Serializable]
    public class RectControlModel
    {
        public Size CtlSize { get; set; }
        public Point CtlLocation { get; set; }
        public BorderStyle BorderStyle { get; set; }
        public Color LineColor { get; set; }
        public int LineWidth { get; set; }


    }
}
