﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Drawing.Drawing2D;
using System.Collections;

namespace SLabelDesign.DesignerControl
{
    public partial class UILabelControl : UserControl, System.ComponentModel.ICustomTypeDescriptor
    {
        // 设置属性显示的白名单
        List<string> whiteNameList = new List<string>() { "Size", "Location", "BorderStyle", "font","LblText", "LblAlignment" }.ConvertAll(d => d.ToLower());


        public UILabelControl()
        {
            InitializeComponent();

            AutoDragComponent adcomponent = new AutoDragComponent();
            adcomponent.initProperty(this);
        }
        public float PreScalingX { get; set; } = 1.0F;
        private float scalingX = 1.0F; //缩放比例
        public float ScalingX { get { return scalingX; } set {  scalingX = value; Refresh(); } }

        public float PreScalingY { get; set; } = 1.0F;
        private float scalingY = 1.0F; //缩放比例
        public float ScalingY { get { return scalingY; } set {  scalingY = value; Refresh(); } }

        private string lblText = "";
        [Category("其他设置")]
        [Description("文本设置")]
        [DisplayName("文本")]
        public string LblText { get { return lblText; } set { lblText = value; Refresh(); } }

        private ContentAlignment lblAlignment = ContentAlignment.TopLeft;
        [Category("其他设置")]
        [Description("文本位置")]
        [DisplayName("文本位置")]
        public ContentAlignment LblAlignment { get { return lblAlignment; } set { this.lblAlignment = value;Refresh(); } }

        private void UILabelControl_Paint(object sender, PaintEventArgs e)
        {
            
            Graphics gctl = this.CreateGraphics();
            gctl.Clear(this.BackColor);

            Point l = this.Location;
            Matrix matrix = new Matrix();
            matrix.Scale(ScalingX, ScalingY);
            gctl.Transform = matrix;

            if (LblAlignment == ContentAlignment.TopLeft)
            {
                gctl.DrawString(this.lblText, this.Font, Brushes.Black, 0, 0);
            }
            if (LblAlignment == ContentAlignment.TopCenter)
            {
                SizeF sizeF = gctl.MeasureString(LblText, this.Font);
                float relayWidth = (int)(sizeF.Width * scalingX );
                int x = (int)((this.Width - relayWidth) / 2);
                gctl.DrawString(this.lblText, this.Font, Brushes.Black, x / scalingX, 0);
            }
            if (LblAlignment == ContentAlignment.TopRight)
            {
                SizeF sizeF = gctl.MeasureString(LblText, this.Font);
                float relayWidth = (int)(sizeF.Width * scalingX );
                int x = (int)((this.Width - relayWidth));
                gctl.DrawString(this.lblText, this.Font, Brushes.Black, x / scalingX, 0);
            }

            if (LblAlignment == ContentAlignment.MiddleLeft)
            {
                SizeF sizeF = gctl.MeasureString(LblText, this.Font);
     
                float realyHeight = (int)(sizeF.Height * scalingY);
                int y= (int)((this.Height - realyHeight) / 2);
                gctl.DrawString(this.lblText, this.Font, Brushes.Black, 0, y/ScalingY);

            }
            if (LblAlignment == ContentAlignment.MiddleCenter)
            {
                SizeF sizeF = gctl.MeasureString(LblText, this.Font);

                float relayWidth = (int)(sizeF.Width * scalingX);
                int x = (int)((this.Width - relayWidth) / 2);

                float realyHeight = (int)(sizeF.Height * scalingY);
                int y = (int)((this.Height - realyHeight) / 2);

                gctl.DrawString(this.lblText, this.Font, Brushes.Black, x / scalingX, y / ScalingY);

            }
            if (LblAlignment == ContentAlignment.MiddleRight)
            {
                SizeF sizeF = gctl.MeasureString(LblText, this.Font);

                float relayWidth = (int)(sizeF.Width * scalingX);
                int x = (int)((this.Width - relayWidth));

                float realyHeight = (int)(sizeF.Height * scalingY);
                float y = (int)((this.Height - realyHeight) / 2);

                gctl.DrawString(this.lblText, this.Font, Brushes.Black, x / scalingX, y / ScalingY);

            }

            if (LblAlignment == ContentAlignment.BottomLeft)
            {
                SizeF sizeF = gctl.MeasureString(LblText, this.Font);

                float realyHeight = (int)(sizeF.Height * scalingY);
                float y = (int)((this.Height - realyHeight));

                gctl.DrawString(this.lblText, this.Font, Brushes.Black, 0, y / ScalingY);
            }

            if (LblAlignment == ContentAlignment.BottomCenter)
            {
                SizeF sizeF = gctl.MeasureString(LblText, this.Font);
                float relayWidth = (int)(sizeF.Width * scalingX);
                int x = (int)((this.Width - relayWidth) / 2);

                float realyHeight = (int)(sizeF.Height * scalingY);
                float y = (int)((this.Height - realyHeight));

                gctl.DrawString(this.lblText, this.Font, Brushes.Black, x/scalingX, y / ScalingY);


            }

            if (LblAlignment == ContentAlignment.BottomRight)
            {
                SizeF sizeF = gctl.MeasureString(LblText, this.Font);
                float relayWidth = (int)(sizeF.Width * scalingX);
                int x = (int)((this.Width - relayWidth));

                float realyHeight = (int)(sizeF.Height * scalingY);
                float y = (int)((this.Height - realyHeight));

                gctl.DrawString(this.lblText, this.Font, Brushes.Black, x / scalingX, y / ScalingY);
            }
            gctl.Dispose();

         }

        private void UILabelControl_Resize(object sender, EventArgs e)
        {
            Refresh();
        }

        #region 显示实现接口
        AttributeCollection ICustomTypeDescriptor.GetAttributes()
        {
            return TypeDescriptor.GetAttributes(this, true);
        }

        string ICustomTypeDescriptor.GetClassName()
        {
            return TypeDescriptor.GetClassName(this, true);
        }

        string ICustomTypeDescriptor.GetComponentName()
        {
            return TypeDescriptor.GetComponentName(this, true);
        }

        TypeConverter ICustomTypeDescriptor.GetConverter()
        {
            return TypeDescriptor.GetConverter(this, true);
        }

        EventDescriptor ICustomTypeDescriptor.GetDefaultEvent()
        {
            return TypeDescriptor.GetDefaultEvent(this, true);
        }

        PropertyDescriptor ICustomTypeDescriptor.GetDefaultProperty()
        {
            return TypeDescriptor.GetDefaultProperty(this, true);
        }

        object ICustomTypeDescriptor.GetEditor(Type editorBaseType)
        {
            return TypeDescriptor.GetEditor(this, editorBaseType, true);
        }

        EventDescriptorCollection ICustomTypeDescriptor.GetEvents()
        {
            return TypeDescriptor.GetEvents(this, true);
        }

        EventDescriptorCollection ICustomTypeDescriptor.GetEvents(Attribute[] attributes)
        {
            return TypeDescriptor.GetEvents(this, attributes, true);
        }

        PropertyDescriptorCollection ICustomTypeDescriptor.GetProperties()
        {
            return ((ICustomTypeDescriptor)this).GetProperties(new Attribute[0]);
        }

        PropertyDescriptorCollection ICustomTypeDescriptor.GetProperties(Attribute[] attributes)
        {
            ArrayList props = new ArrayList();
            Type thisType = this.GetType();

            PropertyDescriptorCollection propsDC = TypeDescriptor.GetProperties(thisType);

            foreach (PropertyDescriptor dc in propsDC)
            {
                if (whiteNameList.Contains(dc.Name.ToLower()))
                {
                    props.Add(dc);
                }
            }

            PropertyDescriptor[] propArray =
            (PropertyDescriptor[])props.ToArray(typeof(PropertyDescriptor));
            return new PropertyDescriptorCollection(propArray);
        }

        object ICustomTypeDescriptor.GetPropertyOwner(PropertyDescriptor pd)
        {
            return this;
        }



        #endregion


    }

    [Serializable]
    public class UILabelControlModel
    {
        public Size CtlSize { get; set; }
        public Point CtlLocation { get; set; }
        public string LabelTXT { get; set; }
        public BorderStyle BorderStyle { get; set; }
        public Font TXTFont { get; set; }
        public ContentAlignment TXTAlign { get; set; }
    }
}
