﻿using SLabelDesign.DesignerControl;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Printing;
using System.Reflection;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace SLabelDesign
{
    public partial class FormMain : Form
    {
        // 软件版本号
        public static string softVersion= System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.ToString();
        // 正在编辑的标签实体
        private LabelModel global_LabelModel = new LabelModel();

        public FormMain()
        {
            InitializeComponent();
            this.Text = "LabelDesigner " + softVersion;

            // 初始化画布
            ChangeLabelSizeAndScaling(global_LabelModel);

            // 处理画布鼠标拖动画虚线框事件、鼠标点击事件等。
            HelperRectangleSeletControl hRectSCtl = new HelperRectangleSeletControl();
            hRectSCtl.initProperty(panel_drawpaper);
            hRectSCtl.handle_RectangleSeletedControls += 
                new HelperRectangleSeletControl.Handle_RectangleSeletedControls((Control ctl)=> {

                    MultiSelectControl(ctl);

                });
        }

        // GDI函数，用于将Panel转换成图片
        [System.Runtime.InteropServices.DllImportAttribute("gdi32.dll ")]
        private static extern bool BitBlt(
        IntPtr hdcDest, // handle to destination DC 
        int nXDest, // x-coord of destination upper-left corner 
        int nYDest, // y-coord of destination upper-left corner 
        int nWidth, // width of destination rectangle 
        int nHeight, // height of destination rectangle 
        IntPtr hdcSrc, // handle to source DC 
        int nXSrc, // x-coordinate of source upper-left corner 
        int nYSrc, // y-coordinate of source upper-left corner 
        System.Int32 dwRop // raster operation code 
        );

        #region 工具栏按钮事件
        // 新建
        private void btn_newLabel_Click(object sender, EventArgs e)
        {
            new FormMain().Show();
        }
        // 打开
        private void btn_open_Click(object sender, EventArgs e)
        {
            OpenFileDialog saveFile = new OpenFileDialog();
            saveFile.Title = "请选择保存文件路径";
            saveFile.Filter = "LabelDesigner(*.lbld)|*.lbld";

            if (saveFile.ShowDialog() != DialogResult.OK) { return;}


            global_LabelModel = SerializeClassHelper.DeserializeObject(saveFile.FileName) as LabelModel;
            global_LabelModel.labelFilePath = saveFile.FileName;
            tabPage_main.Text = global_LabelModel.labelFilePath;

            panel_drawpaper.Controls.Clear();
            ChangeLabelSizeAndScaling(global_LabelModel);
            
            foreach (KeyValuePair<string, Object> keypair in global_LabelModel.ctlList)
            {
                string keypairsplit = keypair.Key.Split('-')[1];
                if (keypairsplit.Equals("QRCodeControl"))
                {
                    QRCodeControlModel model = Newtonsoft.Json.JsonConvert.DeserializeObject<QRCodeControlModel>(keypair.Value.ToString());

                    QRCodeControl qrcc = GenCodeControl(model);
                   

                    panel_drawpaper.Controls.Add(qrcc);
                    continue;
                }

                if (keypairsplit.Equals("LabelControl"))
                {
                    LabelControlModel model = Newtonsoft.Json.JsonConvert.DeserializeObject<LabelControlModel>(keypair.Value.ToString());

                    LabelControl lblc = GenLabelControl(model);
                    

                    panel_drawpaper.Controls.Add(lblc);
                    continue;
                }

                if (keypairsplit.Equals("PictureControl"))
                {
                    
                    PictureControlModel model = Newtonsoft.Json.JsonConvert.DeserializeObject<PictureControlModel>(keypair.Value.ToString());

                    PictureControl pctl = GenPictureControl(model);
                    
                    panel_drawpaper.Controls.Add(pctl);
                    continue;
                }

                if (keypairsplit.Equals("LineControl"))
                {
                    LineControlModel model= Newtonsoft.Json.JsonConvert.DeserializeObject<LineControlModel>(keypair.Value.ToString());

                    LineControl lctl = GenLineControl(model);
                    
                    panel_drawpaper.Controls.Add(lctl);
                    continue;

                }

                if (keypairsplit.Equals("RectControl"))
                {
                    RectControlModel model= Newtonsoft.Json.JsonConvert.DeserializeObject<RectControlModel>(keypair.Value.ToString());

                    RectControl rect = GenRectUserControl(model);


                    panel_drawpaper.Controls.Add(rect);
                    continue;
                }

                if (keypairsplit.Equals("UILabelControl")) {
                    UILabelControlModel model = Newtonsoft.Json.JsonConvert.DeserializeObject<UILabelControlModel>(keypair.Value.ToString());

                    UILabelControl UILbl = GenUILabelControl(model);

                    panel_drawpaper.Controls.Add(UILbl);

                    continue;
                }
            }
        }

        // 保存
        private void btn_save_Click(object sender, EventArgs e)
        {
            string saveFileName = global_LabelModel.labelFilePath;
            if (global_LabelModel.labelFilePath == null)
            {
                SaveFileDialog saveFile = new SaveFileDialog();  //实例化保存文件对话框对象
                saveFile.Title = "请选择保存文件路径";
                saveFile.Filter = "LabelDesigner 文档(*.lbld)|*.lbld";
                saveFile.OverwritePrompt = true;  //是否覆盖当前文件
                saveFile.RestoreDirectory = true;  //还原目录
                if (saveFile.ShowDialog() != DialogResult.OK) { return; }

                saveFileName = saveFile.FileName;

            }


            global_LabelModel.labelFilePath = saveFileName;
            global_LabelModel.ctlList.Clear();
            // 遍历所有控件，保存他们的属性
            int i = 0;
            foreach (Control ctl in this.panel_drawpaper.Controls)
            {
                if (ctl is QRCodeControl)
                {
                    QRCodeControl ctltmp=(ctl as QRCodeControl);
                    QRCodeControlModel ctlModel = new QRCodeControlModel()
                    {
                        CtlLocation = ctltmp.Location,
                        CtlSize = ctltmp.Size,
                        QRCodeType = ctltmp.QRCodeType,
                        QRCodeStr = ctltmp.QRCodeStr,
                        BorderStyle = ctltmp.BorderStyle
                    };
                    global_LabelModel.ctlList.Add((i++)+"-"+"QRCodeControl", ctlModel);
                    continue;
                }

                if (ctl is LabelControl)
                {
                    LabelControl ctltmp = (ctl as LabelControl);
                    LabelControlModel ctlModel = new LabelControlModel()
                    {
                        CtlLocation = ctltmp.Location,
                        CtlSize = ctltmp.Size,
                        LabelTXT=ctltmp.LabelTXT,
                        BorderStyle=ctltmp.BorderStyle,
                        TXTFont=ctltmp.Font,
                        TXTAlign=ctltmp.TextAlign
                    };
                    global_LabelModel.ctlList.Add((i++)+"-"+"LabelControl", ctlModel);
                    continue;
                }

                if (ctl is PictureControl)
                {
                    HelperImage helperimage = new HelperImage();
                    PictureControl ctltmp = (ctl as PictureControl);
                    PictureControlModel ctlModel = new PictureControlModel()
                    {
                        CtlLocation = ctltmp.Location,
                        CtlSize = ctltmp.Size,
                        CtlBorderStyle = ctltmp.BorderStyle,
                        CtlImage = helperimage.GetBase64FromImage(ctltmp.Image),
                        CtlSizeMode=ctltmp.SizeMode
                    };
                    global_LabelModel.ctlList.Add((i++) + "-" + "PictureControl", ctlModel);
                    continue;
                }

                if(ctl is LineControl)
                {
                    LineControl lctl = (ctl as LineControl);
                    LineControlModel lctlModel = new LineControlModel()
                    {
                        CtlLocation = lctl.Location,
                        CtlSize = lctl.Size,
                        BorderStyle = lctl.BorderStyle,
                        LineColor = lctl.LineColor,
                        LineType = lctl.LineType,
                        LineWidth = lctl.LineWidth
                    };
                    global_LabelModel.ctlList.Add((i++) + "-" + "LineControl", lctlModel);
                    continue;
                }

                if(ctl is RectControl)
                {
                    RectControl rect = (ctl as RectControl);
                    RectControlModel rectModel = new RectControlModel() {
                        CtlLocation = rect.Location,
                        CtlSize = rect.Size,
                        BorderStyle = rect.BorderStyle,
                        LineColor = rect.LineColor,
                        LineWidth = rect.LineWidth
                    };
                    global_LabelModel.ctlList.Add((i++) + "-" + "RectControl", rectModel);
                    continue;
                }

                if(ctl is UILabelControl)
                {
                    UILabelControl UILbl = (ctl as UILabelControl);
                    UILabelControlModel UILblModel = new UILabelControlModel() {
                        CtlLocation = UILbl.Location,
                        CtlSize = UILbl.Size,
                        BorderStyle = UILbl.BorderStyle,
                        LabelTXT=UILbl.LblText,
                        TXTFont=UILbl.Font,
                        TXTAlign=UILbl.LblAlignment
                    };
                    global_LabelModel.ctlList.Add((i++) + "-" + "UILabelControl", UILblModel);
                    continue;
                }
            }

            SerializeClassHelper.SerializeObject(saveFileName, global_LabelModel);
            tabPage_main.Text = saveFileName;

            tabPage_main.Text = global_LabelModel.labelFilePath + " (" + DateTime.Now.ToString("HH:mm:ss") + " 保存成功)";
            //延时 1s 消除提示
            Task.Factory.StartNew(() => {
                Thread.Sleep(1000);
                /*开始执行线程代码*/
                this.Invoke(new MethodInvoker(() => {
                    tabPage_main.Text = global_LabelModel.labelFilePath;
                }));
            });

        }

        // 关闭
        private void btn_close_Click(object sender, EventArgs e)
        {
            global_LabelModel = new LabelModel();
            panel_drawpaper.Controls.Clear();
            tabPage_main.Text = "标签文档 -  未命名";

            ChangeLabelSizeAndScaling(global_LabelModel);
            
        }

        // 放大
        private void btn_bigscaling_Click(object sender, EventArgs e)
        {
            if (global_LabelModel.labelScaling >=8.8f) { return; }
            global_LabelModel.labelScaling = global_LabelModel.labelScaling + (float)0.5;
            ChangeLabelSizeAndScaling(global_LabelModel);
        }
        
        // 缩小
        private void btn_smallScaling_Click(object sender, EventArgs e)
        {
            if (global_LabelModel.labelScaling <= 0.8f) { return; }
            global_LabelModel.labelScaling = global_LabelModel.labelScaling - (float)0.5;
            ChangeLabelSizeAndScaling(global_LabelModel);
        }

        // 打印
        private void btn_print_Click(object sender, EventArgs e)
        {
            StringBuilder zplCmd = new StringBuilder();
            global_LabelModel.labelScaling = 1.0f;
            ChangeLabelSizeAndScaling(global_LabelModel);


            Graphics g1 = panel_drawpaper.CreateGraphics();
            Image myImage = new Bitmap(this.panel_drawpaper.Width, this.panel_drawpaper.Height, g1);
            Graphics g2 = Graphics.FromImage(myImage);
            IntPtr dc1 = g1.GetHdc();
            IntPtr dc2 = g2.GetHdc();
            BitBlt(dc2, 0, 0, this.panel_drawpaper.Width, this.panel_drawpaper.Height, dc1, 0, 0, 13369376);
            g1.ReleaseHdc(dc1);
            g2.ReleaseHdc(dc2);

            int to = 0;
            int row = 0;
            string data = ZebraUnity.BitmapToHex(myImage, out to, out row);
            string zpl = string.Format("~DGR:Temp0.GRF,{0},{1},{2}", to, row, data);


            //zplCmd.Append("^XA\r\n");
            //zplCmd.Append("^LH0,0\r\n");
            //zplCmd.Append("^LL800\r\n");
            //zplCmd.Append("^PW800\r\n");
            //zplCmd.Append(zpl+ "^FO0,0^XGR:Temp0.GRF,1,1^FS\r\n");
            //zplCmd.Append("^XZ");

            zplCmd.Append(zpl + "\r\n");

            zplCmd.Append("^XA\r\n");
            zplCmd.Append("^LH0,0\r\n");
            zplCmd.Append("^LL400\r\n");
            zplCmd.Append("^PW800\r\n");
            zplCmd.Append("^FO0,0^XGR:Temp0.GRF,2,2^FS\r\n");
            zplCmd.Append("^XZ");


            new FormPrinter(zplCmd.ToString()).ShowDialog();

        }

        #endregion

        #region 工具箱控件区域
       
        // 点击控件，然后用天蓝色边框突出显示
        private void SingleSelectControl(object sender)
        {
            propertyGrid1.SelectedObject = sender;
            Pen pp = new Pen(Color.SteelBlue);
            pp.Width = 4;
            Control ctl = (sender as Control);
            Graphics g = Graphics.FromHwnd(panel_drawpaper.Handle);
            g.Clear(panel_drawpaper.BackColor);
            g.DrawEllipse(pp, ctl.Location.X - 4, ctl.Location.Y - 4, 4, 4);
            g.DrawEllipse(pp, ctl.Location.X + ctl.Width, ctl.Location.Y - 4, 4, 4);
            g.DrawEllipse(pp, ctl.Location.X - 4, ctl.Location.Y + ctl.Height, 4, 4);
            g.DrawEllipse(pp, ctl.Location.X + ctl.Width, ctl.Location.Y + ctl.Height, 4, 4);
            g.DrawRectangle(pp, ctl.Location.X, ctl.Location.Y, ctl.Width, ctl.Height);

            g.Dispose();
        }
        // 框选控件，然后用天蓝色边框突出显示
        private void MultiSelectControl(object sender)
        {
            Pen pp = new Pen(Color.SteelBlue);
            pp.Width = 4;
            Control ctl = (sender as Control);
            Graphics g = Graphics.FromHwnd(panel_drawpaper.Handle);

            g.DrawEllipse(pp, ctl.Location.X - 4, ctl.Location.Y - 4, 4, 4);
            g.DrawEllipse(pp, ctl.Location.X + ctl.Width, ctl.Location.Y - 4, 4, 4);
            g.DrawEllipse(pp, ctl.Location.X - 4, ctl.Location.Y + ctl.Height, 4, 4);
            g.DrawEllipse(pp, ctl.Location.X + ctl.Width, ctl.Location.Y + ctl.Height, 4, 4);
            g.DrawRectangle(pp, ctl.Location.X, ctl.Location.Y, ctl.Width, ctl.Height);

            g.Dispose();
        }

        // 文本标签
        private void btn_labelControl_Click(object sender, EventArgs e)
        {
            panel_drawpaper.Controls.Add(GenUILabelControl(null));
        }
        private LabelControl GenLabelControl(LabelControlModel model)
        {
            LabelControl lblC = new LabelControl();
            lblC.Text = "标签1";

            if (model != null)
            {
                lblC.Size = model.CtlSize;
                lblC.Location = model.CtlLocation;
                lblC.LabelTXT = model.LabelTXT;
                lblC.BorderStyle = model.BorderStyle;
                lblC.Font = model.TXTFont;
                lblC.TextAlign = model.TXTAlign;
            }
            ContextMenuStrip controlContextMenuStrip = new ContextMenuStrip();
            controlContextMenuStrip.Items.Add("置于顶层", null, BringToFrontControl_ToolStripMenuItem_Click);
            controlContextMenuStrip.Items.Add("置于底层", null, BringToBackControl_ToolStripMenuItem_Click);
            controlContextMenuStrip.Items.Add("删除元素", null, DeleteControl_ToolStripMenuItem_Click);
            controlContextMenuStrip.Items.Add("属性", null, Attribute_ToolStripMenuItem_Click);

            lblC.ContextMenuStrip = controlContextMenuStrip;
            lblC.LocationChanged += (object sender, EventArgs args) => {
                propertyGrid1.SelectedObject = sender;
            };
            lblC.Resize += (object sender, EventArgs args) =>
            {
                propertyGrid1.SelectedObject = sender;
            };

            lblC.MouseClick+=(object sender, MouseEventArgs args) => {
                SingleSelectControl(sender);
            };

            return lblC;
        }

        private UILabelControl GenUILabelControl(UILabelControlModel model)
        {
            UILabelControl uilbl = new UILabelControl();
            uilbl.LblText = "文字样本";
            uilbl.Location = new Point(10, 10);
            uilbl.Size = new Size(100, 30);
            uilbl.BorderStyle = BorderStyle.None;
            uilbl.ScalingX = global_LabelModel.labelScaling;
            uilbl.PreScalingX = global_LabelModel.preLabelScaling;
            uilbl.ScalingY = global_LabelModel.labelScaling;
            uilbl.PreScalingY = global_LabelModel.preLabelScaling;

            if (model != null)
            {
                uilbl.LblText = model.LabelTXT;
                /**字体设置要优先位置设置，不然会有Bug。原因：windows在设置字体时候，会调整控件的位置和大小。**/
                uilbl.Font = model.TXTFont;


                uilbl.LblAlignment = model.TXTAlign;
                uilbl.Size = model.CtlSize;
                uilbl.Location = model.CtlLocation;
                uilbl.BorderStyle = model.BorderStyle;
            }
            
            ContextMenuStrip controlContextMenuStrip = new ContextMenuStrip();
            controlContextMenuStrip.Items.Add("置于顶层", null, BringToFrontControl_ToolStripMenuItem_Click);
            controlContextMenuStrip.Items.Add("置于底层", null, BringToBackControl_ToolStripMenuItem_Click);
            controlContextMenuStrip.Items.Add("删除元素", null, DeleteControl_ToolStripMenuItem_Click);
            controlContextMenuStrip.Items.Add("属性", null, Attribute_ToolStripMenuItem_Click);

            uilbl.ContextMenuStrip = controlContextMenuStrip;
            uilbl.LocationChanged += (object sender, EventArgs args) => {
                
                propertyGrid1.SelectedObject = sender;
                SingleSelectControl(sender);

            };
            uilbl.Resize += (object sender, EventArgs args) =>
            {
                propertyGrid1.SelectedObject = sender;
                SingleSelectControl(sender);
            };
            
            uilbl.MouseClick += (object sender, MouseEventArgs args) => {
                SingleSelectControl(sender);
            };

            return uilbl;
        }
       
        // 条码标签
        private void btn_codeControl_Click(object sender, EventArgs e)
        { 
            panel_drawpaper.Controls.Add(GenCodeControl(null));
        }
        private QRCodeControl GenCodeControl(QRCodeControlModel model)
        {
            QRCodeControl qrcctl = new QRCodeControl();
            qrcctl.QRCodeStr = "Hello,QR";

            if (model != null)
            {
                qrcctl.Size = model.CtlSize;
                qrcctl.Location = model.CtlLocation;
                qrcctl.QRCodeType = model.QRCodeType;
                qrcctl.QRCodeStr = model.QRCodeStr;
                qrcctl.BorderStyle = model.BorderStyle;
            }

            ContextMenuStrip controlContextMenuStrip = new ContextMenuStrip();
            controlContextMenuStrip.Items.Add("置于顶层", null, BringToFrontControl_ToolStripMenuItem_Click);
            controlContextMenuStrip.Items.Add("置于底层", null, BringToBackControl_ToolStripMenuItem_Click);
            controlContextMenuStrip.Items.Add("删除元素", null, DeleteControl_ToolStripMenuItem_Click);
            controlContextMenuStrip.Items.Add("属性", null, Attribute_ToolStripMenuItem_Click);

            qrcctl.ContextMenuStrip = controlContextMenuStrip;
            qrcctl.LocationChanged += (object sender, EventArgs args) => {
                propertyGrid1.SelectedObject = sender;
                SingleSelectControl(sender);
            };
            qrcctl.Resize += (object sender, EventArgs args) =>
            {
                propertyGrid1.SelectedObject = sender;
                SingleSelectControl(sender);
            };
            qrcctl.MouseClick += (object sender, MouseEventArgs args) => {
                SingleSelectControl(sender);
            };

            return qrcctl;
        }

        // 图片标签
        private void btn_pictureControl_Click(object sender, EventArgs e)
        {
            panel_drawpaper.Controls.Add(GenPictureControl(null));
        }
        private PictureControl GenPictureControl(PictureControlModel model)
        {
            PictureControl picctl = new PictureControl();

            if (model != null)
            {
                HelperImage helperimage = new HelperImage();

                picctl.Size = model.CtlSize;
                picctl.Location = model.CtlLocation;
                picctl.BorderStyle = model.CtlBorderStyle;
                picctl.Image = helperimage.GetImageFromBase64(model.CtlImage);
                picctl.SizeMode = model.CtlSizeMode;
            }

            ContextMenuStrip controlContextMenuStrip = new ContextMenuStrip();
            controlContextMenuStrip.Items.Add("置于顶层", null, BringToFrontControl_ToolStripMenuItem_Click);
            controlContextMenuStrip.Items.Add("置于底层", null, BringToBackControl_ToolStripMenuItem_Click);
            controlContextMenuStrip.Items.Add("删除元素", null, DeleteControl_ToolStripMenuItem_Click);
            controlContextMenuStrip.Items.Add("属性", null, Attribute_ToolStripMenuItem_Click);

            picctl.ContextMenuStrip = controlContextMenuStrip;
            picctl.LocationChanged += (object sender, EventArgs args) => {
                propertyGrid1.SelectedObject = sender;
                SingleSelectControl(sender);
            };
            picctl.Resize += (object sender, EventArgs args) =>
            {
                propertyGrid1.SelectedObject = sender;
                SingleSelectControl(sender);
            };
            picctl.MouseClick += (object sender, MouseEventArgs args) => {
                SingleSelectControl(sender);
            };


            return picctl;
        }

        // 直线标签
        private void btn_line_Click(object sender, EventArgs e)
        {
            panel_drawpaper.Controls.Add(GenLineControl(null));
        }
        private LineControl GenLineControl(LineControlModel model)
        {
            LineControl lctl = new LineControl();
            lctl.Location = new Point(10,10);
            lctl.Size = new Size(100,30);

            if (model != null)
            {
                lctl.Size = model.CtlSize;
                lctl.Location = model.CtlLocation;
                lctl.BorderStyle = model.BorderStyle;
                lctl.LineColor = model.LineColor;
                lctl.LineType = model.LineType;
                lctl.LineWidth = model.LineWidth;
            }

            ContextMenuStrip controlContextMenuStrip = new ContextMenuStrip();
            controlContextMenuStrip.Items.Add("置于顶层", null, BringToFrontControl_ToolStripMenuItem_Click);
            controlContextMenuStrip.Items.Add("置于底层", null, BringToBackControl_ToolStripMenuItem_Click);
            controlContextMenuStrip.Items.Add("删除元素", null, DeleteControl_ToolStripMenuItem_Click);
            controlContextMenuStrip.Items.Add("属性", null, Attribute_ToolStripMenuItem_Click);

            lctl.ContextMenuStrip = controlContextMenuStrip;
            lctl.LocationChanged += (object sender, EventArgs args) => {
                propertyGrid1.SelectedObject = sender;
                SingleSelectControl(sender);
            };
            lctl.Resize += (object sender, EventArgs args) =>
            {
                propertyGrid1.SelectedObject = sender;
                SingleSelectControl(sender);
            };
            lctl.MouseClick += (object sender, MouseEventArgs args) => {
                SingleSelectControl(sender);
            };


            return lctl;
        }

        // 矩形标签
        private void btn_rect_Click(object sender, EventArgs e)
        {
            panel_drawpaper.Controls.Add(GenRectUserControl(null));
        }

        private RectControl GenRectUserControl(RectControlModel model)
        {
            RectControl rect = new RectControl();

            rect.Location = new Point(10, 20);
            rect.Size = new Size(200, 40);

            if (model != null)
            {
                rect.Size = model.CtlSize;
                rect.Location = model.CtlLocation;
                rect.BorderStyle = model.BorderStyle;
                rect.LineColor = model.LineColor;
                rect.LineWidth = model.LineWidth;
            }

            ContextMenuStrip controlContextMenuStrip = new ContextMenuStrip();
            controlContextMenuStrip.Items.Add("置于顶层", null, BringToFrontControl_ToolStripMenuItem_Click);
            controlContextMenuStrip.Items.Add("置于底层", null, BringToBackControl_ToolStripMenuItem_Click);
            controlContextMenuStrip.Items.Add("删除元素", null, DeleteControl_ToolStripMenuItem_Click);
            controlContextMenuStrip.Items.Add("属性", null, Attribute_ToolStripMenuItem_Click);

            rect.ContextMenuStrip = controlContextMenuStrip;
            rect.LocationChanged += (object sender, EventArgs args) => {
                propertyGrid1.SelectedObject = sender;
                SingleSelectControl(sender);
            };
            rect.Resize += (object sender, EventArgs args) =>
            {
                propertyGrid1.SelectedObject = sender;
                SingleSelectControl(sender);
            };
            rect.MouseClick += (object sender, MouseEventArgs args) => {
                SingleSelectControl(sender);
            };


            return rect;
        }

        #endregion 工具箱控件区域

        #region 控件右键菜单事件

        // 控件属性
        private void Attribute_ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Control ctl = (((ToolStripMenuItem)sender).GetCurrentParent() as ContextMenuStrip).SourceControl;
            //MessageBox.Show(ctl.ToString());
            propertyGrid1.SelectedObject = ctl;
        }
        // 删除选中控件
        private void DeleteControl_ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Control ctl = (((ToolStripMenuItem)sender).GetCurrentParent() as ContextMenuStrip).SourceControl;
            panel_drawpaper.Controls.Remove(ctl);
            // 清空画布
            Graphics g = Graphics.FromHwnd(panel_drawpaper.Handle);
            g.Clear(panel_drawpaper.BackColor);
            g.Dispose();

        }
        // 置于顶层
        private void BringToFrontControl_ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Control ctl = (((ToolStripMenuItem)sender).GetCurrentParent() as ContextMenuStrip).SourceControl;
            ctl.BringToFront();
        }
        // 置于底层
        private void BringToBackControl_ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Control ctl = (((ToolStripMenuItem)sender).GetCurrentParent() as ContextMenuStrip).SourceControl;
            ctl.SendToBack();
        }

        #endregion

        #region 画布右键菜单
        private void 标签设置ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormAttributeSetting frmsetting = new FormAttributeSetting(global_LabelModel);
            frmsetting.changeFormMain_DrawPaper += ((int width, int height,float scaling) => {
                global_LabelModel.labelDrawPaperSize = new Size(width,height);
                global_LabelModel.labelScaling = scaling;
                ChangeLabelSizeAndScaling(global_LabelModel);
            });
            frmsetting.ShowDialog();
        }

        private void 放大ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (global_LabelModel.labelScaling >8.8f) { return; }
            global_LabelModel.labelScaling = global_LabelModel.labelScaling + (float)0.5;
            ChangeLabelSizeAndScaling(global_LabelModel);
        }

        private void 缩小ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (global_LabelModel.labelScaling <= 0.8f) { return; }
            global_LabelModel.labelScaling = global_LabelModel.labelScaling - (float)0.5;
            ChangeLabelSizeAndScaling(global_LabelModel);
        }

        private void 正常100ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            global_LabelModel.labelScaling = 1.0f;
            ChangeLabelSizeAndScaling(global_LabelModel);
        }

        private void 正常150ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            global_LabelModel.labelScaling = 1.5f;
            ChangeLabelSizeAndScaling(global_LabelModel);
        }

        private void 正常200ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            global_LabelModel.labelScaling = 2.0f;
            ChangeLabelSizeAndScaling(global_LabelModel);
        }

        // 更改标尺缩放&更改标签大小&更改缩放比例方法
        private void ChangeLabelSizeAndScaling(LabelModel lblm)
        {
            linearScaleX1.Scaling = lblm.labelScaling;
            linearScaleY1.Scaling = lblm.labelScaling;

            double MonitorDPIX = 1.0f;
            using (Graphics graphics = Graphics.FromHwnd(IntPtr.Zero))
            {
                MonitorDPIX = graphics.DpiX;
            }
            // 设置标签大小&缩放比例
            int mm_width = lblm.labelDrawPaperSize.Width;
            int xs_width = (int)((MonitorDPIX * lblm.labelScaling * mm_width) / 25.4);

            int mm_height = lblm.labelDrawPaperSize.Height;
            int xs_height = (int)((MonitorDPIX * lblm.labelScaling * mm_height) / 25.4);

            panel_drawpaper.Size = new Size(xs_width, xs_height);

            // 设置标签中所有元素的大小&缩放比例
            foreach (Control ctl in this.panel_drawpaper.Controls)
            {

                int ctl_X = (int)(ctl.Location.X * lblm.labelScaling / lblm.preLabelScaling);
                int ctl_Y = (int)(ctl.Location.Y * lblm.labelScaling / lblm.preLabelScaling);

                ctl.Location = new Point(ctl_X, ctl_Y);

                int ctl_width= (int)(ctl.Width * lblm.labelScaling / lblm.preLabelScaling);
                int ctl_height = (int)(ctl.Height * lblm.labelScaling / lblm.preLabelScaling);

                ctl.Size = new Size(ctl_width, ctl_height);


                //缩放字体
                //if (ctl is UILabelControl)
                //{
                //    UILabelControl uilbl = (ctl as UILabelControl);
                //    uilbl.ScalingX = lblm.labelScaling;
                //    uilbl.ScalingY = lblm.labelScaling;
                //}
            }

            


            //panel_drawpaper.Scale(new SizeF(lblm.labelScaling, lblm.labelScaling));
            toolStripStatusLabel_scaling.Text = "缩放比例:" + (lblm.labelScaling*100)+"%";

            ClearPanelDrawPaper();
        }

        // 清空画布
        private void ClearPanelDrawPaper()
        {
            // 清空画布
            Graphics g = Graphics.FromHwnd(panel_drawpaper.Handle);
            g.Clear(panel_drawpaper.BackColor);
            g.Dispose();
        }
        // 动态计算字体大小
        private float Calc_Text_size(string s, int width, int height)
        {

            

            float size = 0f;
            size = (float)((width) / s.Length);//1.2是我自己测算出来的
            if (size > height - 3) //3也是自己测算出来的
            {
                size = height - 3;
            }
            return size;
        }
        #endregion

        #region 菜单栏
        private void 新建ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            btn_newLabel_Click(sender,e);
        }

        private void 打开ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            btn_open_Click(sender, e);
        }

        private void 关闭ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            btn_close_Click(sender, e);
        }

        private void 保存ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            btn_save_Click(sender, e);
        }

        private void 退出ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void 关于软件ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("简单的标签设计与打印软件，LabelDesigner 当前版本："+ softVersion + "。");
        }



        #endregion

        #region 控件列表
        private void comboBox_controlsList_DropDown(object sender, EventArgs e)
        {
            comboBox_controlsList.Items.Clear();
            int i = 0;
            foreach (Control ctl in this.panel_drawpaper.Controls)
            {
                string ss=ctl.GetType().Name;
                comboBox_controlsList.Items.Add((i++)+"-"+ctl.GetType().Name);
            }
        }
        #endregion

        #region 测试功能
        private void 打印设置ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            PrintDialog printDialog = new PrintDialog();
            printDialog.Document = printDocument1;
            printDialog.ShowDialog();
        }

        private void 页面设置ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            PageSetupDialog pageSetupDialog = new PageSetupDialog();
            pageSetupDialog.Document = printDocument1;
            pageSetupDialog.ShowDialog();
        }

        private void 打印预览ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            global_LabelModel.labelScaling = 1.0f;
            ChangeLabelSizeAndScaling(global_LabelModel);

            PrintPreviewDialog printPreviewDialog = new PrintPreviewDialog();
            printPreviewDialog.Document = printDocument1;

            try
            {
                printPreviewDialog.ShowDialog();
            }
            catch (Exception excep)
            {
                MessageBox.Show(excep.Message, "打印出错", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void 打印ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            global_LabelModel.labelScaling = 1.0f;
            ChangeLabelSizeAndScaling(global_LabelModel);

            PrintDialog printDialog = new PrintDialog();
            printDialog.Document = printDocument1;

            if (printDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    printDocument1.Print();
                }
                catch (Exception excep)
                {
                    MessageBox.Show(excep.Message, "打印出错", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    printDocument1.PrintController.OnEndPrint(printDocument1, new PrintEventArgs());
                }
            }
        }

        private void printDocument1_PrintPage(object sender, PrintPageEventArgs e)
        {
            Graphics g1 = panel_drawpaper.CreateGraphics();
            Image myImage = new Bitmap(this.panel_drawpaper.Width, this.panel_drawpaper.Height, g1);
            Graphics g2 = Graphics.FromImage(myImage);
            IntPtr dc1 = g1.GetHdc();
            IntPtr dc2 = g2.GetHdc();
            BitBlt(dc2, 0, 0, this.panel_drawpaper.Width, this.panel_drawpaper.Height, dc1, 0, 0, 13369376);
            g1.ReleaseHdc(dc1);
            g2.ReleaseHdc(dc2);

            e.Graphics.DrawImage(myImage, 0, 0);
            e.HasMorePages = false;
        }

        #endregion


    }
}
